﻿Sys.Net.ScriptReferenceExecutor = function()
{
    Sys.Net.ScriptReferenceExecutor.initializeBase(this);

    this._responseAvailable = false;
    this._timedOut = false;
    this._aborted = false;
    this._started = false;
    this._responseData = null;
    this._statusCode = 0;
    this._statusText = null;
    
    this._uniqueKey = null;
    this._timer = null;
    this._scriptElement = null;

}
Sys.Net.ScriptReferenceExecutor.prototype = 
{
    get_responseAvailable : function()
    {
        return this._responseAvailable;
    },
    
    get_timedOut : function()
    {
        return this._timedOut;
    },
    
    get_aborted : function()
    {
        return this._aborted;
    },
    
    get_started : function()
    {
        return this._started;
    },
    
    get_responseData : function()
    {
        return this._responseData;
    },
    
    get_statusCode : function()
    {
        return this._statusCode;
    },
    
    get_statusText : function()
    {
        return this._statusText;
    },
    
    get_xml : function()
    {
        return new XMLDOM(this.get_responseData());
    },
    
    executeRequest : function()
    {
        this._started = true;
        
        var request = this.get_webRequest();
        var serializer = Sys.Serialization.JavaScriptSerializer;
        
        var scriptUrl = request.get_url() + "?";
        scriptUrl += (("headers=") + encodeURIComponent(serializer.serialize(request.get_headers())));
        scriptUrl += ("&body=" + encodeURIComponent(serializer.serialize(request.get_body())));
        
        var uniqueKey = this._uniqueKey = this._generateUniqueKey();
        scriptUrl += ("&uniqueKey=" + encodeURIComponent(serializer.serialize(uniqueKey)));
        Sys.Net.ScriptReferenceExecutor._executors[uniqueKey] = this;
        
        var scriptElement = this._scriptElement = document.createElement("script");
        scriptElement.type = "text/javascript";
        scriptElement.language = "javascript";
        scriptElement.src = scriptUrl;
        document.getElementsByTagName("head")[0].appendChild(scriptElement);
        
        var timeout = request.get_timeout();
        if (timeout > 0)
        {
            this._timer = window.setTimeout(
                Function.createDelegate(this, this._onTimeout), timeout);
        }
        
    },
    
    _onTimeout : function()
    {
        this.complete(null, null, null, false, true, false);
    },
    
    abort : function()
    {
        this.complete(null, null, null, false, false, true);
    },
    
    _generateUniqueKey : function()
    {
        return Math.random().toString();
    },
    
    complete : function(statusCode, statusText, body, responseAvailable, timedOut, aborted)
    {
        this._statusCode = statusCode;
        this._statusText = statusText;
        this._responseData = body;
        this._responseAvailable = responseAvailable;
        this._timedOut = timedOut;
        this._aborted = aborted;
        
        if (this._timer)
        {
            window.clearTimeout(this._timer);
        }
        document.getElementsByTagName("head")[0].removeChild(this._scriptElement);
        delete Sys.Net.ScriptReferenceExecutor._executors[this._uniqueKey];
        
        this.get_webRequest().completed(Sys.EventArgs.Empty);
    }
}
Sys.Net.ScriptReferenceExecutor.registerClass("Sys.Net.ScriptReferenceExecutor", Sys.Net.WebRequestExecutor);
Sys.Net.ScriptReferenceExecutor._executors = new Object();
Sys.Net.ScriptReferenceExecutor.complete = function(uniqueKey, statusCode, statusText, body)
{
    var executor = Sys.Net.ScriptReferenceExecutor._executors[uniqueKey];
    if (executor)
    {
        executor.complete(statusCode, statusText, body, true, false, false);
    }
}